<?php

interface IPastebinBackend
{
	// returns void
	public function save($id, $parserId, $text, $html);
	// returns array(parserId, text, html)
	public function load($id);
	// returns void
	public function log($id, $userId, $time);
}

interface IPastebinParser
{
	// returns html:string
	public function parse($text);
}

class PastebinModel
{
	private
		$backend,
		$id;

	public function __construct(IPastebinBackend $backend)
	{
		$this->backend = $backend;
	}

 	// returns html:string
	public function parse($text, $parserId, $userId)
	{
		$this->id = $this->_hash($text, $parserId);
	    $exist = $this->backend->load($this->id);
	    if (!$exist) {
			$parser = PastebinParserLocator::getParserById($parserId);
			$html = $parser->parse($text, $parserId);
			$this->backend->save($this->id, $parserId, $text, $html);
			$this->backend->log($this->id, $userId, microtime(TRUE));
		} else {
			$html = $exist['html'];
		}
		return $html;
	}

	// returns id:string
	public function lastId()
	{
		return $this->id;
	}

	private function _hash($text, $parserId)
	{
		return md5(serialize(array($text, $parserId)));
	}
}

class PastebinParserLocator
{
	private static
		$classes,
		$objects;

	public static function init(array $classes)
	{
		self::$classes = $classes;
	}

	// returns parser:IPastebinParser
	public static function getParserById($parserId)
	{
		if (isset(self::$objects[$parserId])) {
			return self::$objects[$parserId];
		}
		if (!isset(self::$classes[$parserId])) {
			throw new Exception('Unknown parser "' . $parserId .'"');
		}
		$class = self::$classes[$parserId];
		return self::$objects[$parserId] = new $class;
	}
}

